"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeAddCmd = exports.UnresolvedDependencyError = exports.EditorIncompatibleError = exports.InvalidPackumentDataError = void 0;
const logger_1 = __importDefault(require("./logger"));
const package_url_1 = require("../domain/package-url");
const editor_version_1 = require("../domain/editor-version");
const package_reference_1 = require("../domain/package-reference");
const scoped_registry_1 = require("../domain/scoped-registry");
const project_manifest_1 = require("../domain/project-manifest");
const packument_resolving_1 = require("../packument-resolving");
const array_utils_1 = require("../utils/array-utils");
const common_errors_1 = require("../common-errors");
const ts_results_es_1 = require("ts-results-es");
const ts_custom_error_1 = require("ts-custom-error");
const error_logging_1 = require("./error-logging");
const packument_1 = require("../domain/packument");
class InvalidPackumentDataError extends ts_custom_error_1.CustomError {
    constructor(issue) {
        super("A packument object was malformed.");
        this.issue = issue;
        this._class = "InvalidPackumentDataError";
    }
}
exports.InvalidPackumentDataError = InvalidPackumentDataError;
class EditorIncompatibleError extends ts_custom_error_1.CustomError {
    constructor() {
        super("A packuments target editor-version was not compatible with the installed editor-version.");
        this._class = "EditorIncompatibleError";
    }
}
exports.EditorIncompatibleError = EditorIncompatibleError;
class UnresolvedDependencyError extends ts_custom_error_1.CustomError {
    constructor() {
        super("A packuments dependency could not be resolved.");
        this._class = "UnresolvedDependencyError";
    }
}
exports.UnresolvedDependencyError = UnresolvedDependencyError;
/**
 * Makes a {@link AddCmd} function.
 */
function makeAddCmd(parseEnv, resolveRemovePackument, resolveDependencies, loadProjectManifest, writeProjectManifest) {
    return async (pkgs, options) => {
        if (!Array.isArray(pkgs))
            pkgs = [pkgs];
        // parse env
        const envResult = await parseEnv(options);
        if (envResult.isErr())
            return envResult;
        const env = envResult.value;
        if (typeof env.editorVersion === "string")
            logger_1.default.warn("editor.version", `${env.editorVersion} is unknown, the editor version check is disabled`);
        const tryAddToManifest = async function (manifest, pkg) {
            // is upstream package flag
            let isUpstreamPackage = false;
            // parse name
            const [name, requestedVersion] = (0, package_reference_1.splitPackageReference)(pkg);
            // packages that added to scope registry
            const pkgsInScope = Array.of();
            let versionToAdd = requestedVersion;
            if (requestedVersion === undefined || !(0, package_url_1.isPackageUrl)(requestedVersion)) {
                let resolveResult = await resolveRemovePackument(name, requestedVersion, env.registry).promise;
                if (resolveResult.isErr() && env.upstream) {
                    resolveResult = await resolveRemovePackument(name, requestedVersion, env.upstreamRegistry).promise;
                    if (resolveResult.isOk())
                        isUpstreamPackage = true;
                }
                if (resolveResult.isErr()) {
                    if (resolveResult.error instanceof common_errors_1.PackumentNotFoundError)
                        logger_1.default.error("404", `package not found: ${name}`);
                    else if (resolveResult.error instanceof packument_resolving_1.VersionNotFoundError) {
                        const versionList = [...resolveResult.error.availableVersions]
                            .reverse()
                            .join(", ");
                        logger_1.default.warn("404", `version ${resolveResult.error.requestedVersion} is not a valid choice of: ${versionList}`);
                    }
                    return resolveResult;
                }
                const packumentVersion = resolveResult.value.packumentVersion;
                versionToAdd = packumentVersion.version;
                const targetEditorVersionResult = (0, packument_1.tryGetTargetEditorVersionFor)(packumentVersion);
                if (targetEditorVersionResult.isErr()) {
                    logger_1.default.warn("package.unity", `${targetEditorVersionResult.error.versionString} is not valid`);
                    if (!options.force) {
                        logger_1.default.notice("suggest", "contact the package author to fix the issue, or run with option -f to ignore the warning");
                        return (0, ts_results_es_1.Err)(new InvalidPackumentDataError("Editor-version not valid."));
                    }
                }
                else {
                    const targetEditorVersion = targetEditorVersionResult.value;
                    // verify editor version
                    if (targetEditorVersion !== null &&
                        typeof env.editorVersion !== "string" &&
                        (0, editor_version_1.compareEditorVersion)(env.editorVersion, targetEditorVersion) < 0) {
                        logger_1.default.warn("editor.version", `requires ${targetEditorVersion} but found ${(0, editor_version_1.stringifyEditorVersion)(env.editorVersion)}`);
                        if (!options.force) {
                            logger_1.default.notice("suggest", `upgrade the editor to ${targetEditorVersion}, or run with option -f to ignore the warning`);
                            return (0, ts_results_es_1.Err)(new EditorIncompatibleError());
                        }
                    }
                }
                // pkgsInScope
                if (!isUpstreamPackage) {
                    logger_1.default.verbose("dependency", `fetch: ${(0, package_reference_1.makePackageReference)(name, requestedVersion)}`);
                    const [depsValid, depsInvalid] = await resolveDependencies(env.registry, env.upstreamRegistry, name, requestedVersion, true);
                    // add depsValid to pkgsInScope.
                    depsValid
                        .filter((x) => !x.upstream && !x.internal)
                        .map((x) => x.name)
                        .forEach((name) => pkgsInScope.push(name));
                    // print suggestion for depsInvalid
                    let isAnyDependencyUnresolved = false;
                    depsInvalid.forEach((depObj) => {
                        if (depObj.reason instanceof common_errors_1.PackumentNotFoundError ||
                            depObj.reason instanceof packument_resolving_1.VersionNotFoundError) {
                            // Not sure why it thinks the manifest can be null here.
                            const resolvedVersion = manifest.dependencies[depObj.name];
                            const wasResolved = Boolean(resolvedVersion);
                            if (!wasResolved) {
                                isAnyDependencyUnresolved = true;
                                if (depObj.reason instanceof packument_resolving_1.VersionNotFoundError)
                                    logger_1.default.notice("suggest", `to install ${(0, package_reference_1.makePackageReference)(depObj.name, depObj.reason.requestedVersion)} or a replaceable version manually`);
                            }
                        }
                    });
                    if (isAnyDependencyUnresolved) {
                        if (!options.force) {
                            logger_1.default.error("missing dependencies", "please resolve the issue or run with option -f to ignore the warning");
                            return (0, ts_results_es_1.Err)(new UnresolvedDependencyError());
                        }
                    }
                }
                else
                    pkgsInScope.push(name);
            }
            // add to dependencies
            const oldVersion = manifest.dependencies[name];
            // Whether a change was made that requires overwriting the manifest
            let dirty = false;
            // I am not sure why we need this assertion. I'm pretty sure
            // code-logic ensures the correct type.
            manifest = (0, project_manifest_1.addDependency)(manifest, name, versionToAdd);
            if (!oldVersion) {
                // Log the added package
                logger_1.default.notice("manifest", `added ${(0, package_reference_1.makePackageReference)(name, versionToAdd)}`);
                dirty = true;
            }
            else if (oldVersion !== versionToAdd) {
                // Log the modified package version
                logger_1.default.notice("manifest", `modified ${name} ${oldVersion} => ${versionToAdd}`);
                dirty = true;
            }
            else {
                // Log the existed package
                logger_1.default.notice("manifest", `existed ${(0, package_reference_1.makePackageReference)(name, versionToAdd)}`);
            }
            if (!isUpstreamPackage && pkgsInScope.length > 0) {
                manifest = (0, project_manifest_1.mapScopedRegistry)(manifest, env.registry.url, (initial) => {
                    let updated = initial ?? (0, scoped_registry_1.makeEmptyScopedRegistryFor)(env.registry.url);
                    updated = pkgsInScope.reduce(scoped_registry_1.addScope, updated);
                    dirty =
                        !(0, array_utils_1.areArraysEqual)(updated.scopes, initial?.scopes ?? []) || dirty;
                    return updated;
                });
            }
            if (options.test)
                manifest = (0, project_manifest_1.addTestable)(manifest, name);
            return (0, ts_results_es_1.Ok)([manifest, dirty]);
        };
        // load manifest
        const loadResult = await loadProjectManifest(env.cwd).promise;
        if (loadResult.isErr()) {
            (0, error_logging_1.logManifestLoadError)(loadResult.error);
            return loadResult;
        }
        let manifest = loadResult.value;
        // add
        let dirty = false;
        for (const pkg of pkgs) {
            const result = await tryAddToManifest(manifest, pkg);
            if (result.isErr())
                return result;
            const [newManifest, manifestChanged] = result.value;
            if (manifestChanged) {
                manifest = newManifest;
                dirty = true;
            }
        }
        // Save manifest
        if (dirty) {
            const saveResult = await writeProjectManifest(env.cwd, manifest).promise;
            if (saveResult.isErr()) {
                (0, error_logging_1.logManifestSaveError)(saveResult.error);
                return saveResult;
            }
            // print manifest notice
            logger_1.default.notice("", "please open Unity project to apply changes");
        }
        return (0, ts_results_es_1.Ok)(undefined);
    };
}
exports.makeAddCmd = makeAddCmd;
//# sourceMappingURL=cmd-add.js.map