"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeRemoveCmd = void 0;
const logger_1 = __importDefault(require("./logger"));
const package_reference_1 = require("../domain/package-reference");
const scoped_registry_1 = require("../domain/scoped-registry");
const project_manifest_1 = require("../domain/project-manifest");
const ts_results_es_1 = require("ts-results-es");
const common_errors_1 = require("../common-errors");
const error_logging_1 = require("./error-logging");
/**
 * Makes a {@link RemoveCmd} function.
 */
function makeRemoveCmd(parseEnv, loadProjectManifest, writeProjectManifest) {
    return async (pkgs, options) => {
        if (!Array.isArray(pkgs))
            pkgs = [pkgs];
        // parse env
        const envResult = await parseEnv(options);
        if (envResult.isErr())
            return envResult;
        const env = envResult.value;
        const tryRemoveFromManifest = async function (manifest, pkg) {
            // parse name
            if ((0, package_reference_1.hasVersion)(pkg)) {
                const [name] = (0, package_reference_1.splitPackageReference)(pkg);
                logger_1.default.warn("", `please do not specify a version (Write only '${name}').`);
                return (0, ts_results_es_1.Err)(new common_errors_1.PackageWithVersionError());
            }
            // not found array
            const versionInManifest = manifest.dependencies[pkg];
            if (versionInManifest === undefined) {
                logger_1.default.error("404", `package not found: ${pkg}`);
                return (0, ts_results_es_1.Err)(new common_errors_1.PackumentNotFoundError());
            }
            manifest = (0, project_manifest_1.removeDependency)(manifest, pkg);
            manifest = (0, project_manifest_1.mapScopedRegistry)(manifest, env.registry.url, (initial) => {
                if (initial === null)
                    return null;
                return (0, scoped_registry_1.removeScope)(initial, pkg);
            });
            logger_1.default.notice("manifest", `removed ${(0, package_reference_1.makePackageReference)(pkg, versionInManifest)}`);
            return (0, ts_results_es_1.Ok)(manifest);
        };
        // load manifest
        const manifestResult = await loadProjectManifest(env.cwd).promise;
        if (manifestResult.isErr()) {
            (0, error_logging_1.logManifestLoadError)(manifestResult.error);
            return manifestResult;
        }
        let manifest = manifestResult.value;
        // remove
        for (const pkg of pkgs) {
            const result = await tryRemoveFromManifest(manifest, pkg);
            if (result.isErr())
                return result;
            manifest = result.value;
        }
        // save manifest
        const saveResult = await writeProjectManifest(env.cwd, manifest).promise;
        if (saveResult.isErr()) {
            (0, error_logging_1.logManifestSaveError)(saveResult.error);
            return saveResult;
        }
        // print manifest notice
        logger_1.default.notice("", "please open Unity project to apply changes");
        return (0, ts_results_es_1.Ok)(undefined);
    };
}
exports.makeRemoveCmd = makeRemoveCmd;
//# sourceMappingURL=cmd-remove.js.map