"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeViewCmd = void 0;
const chalk_1 = __importDefault(require("chalk"));
const logger_1 = __importDefault(require("./logger"));
const assert_1 = __importDefault(require("assert"));
const packument_1 = require("../domain/packument");
const package_reference_1 = require("../domain/package-reference");
const record_utils_1 = require("../utils/record-utils");
const ts_results_es_1 = require("ts-results-es");
const common_errors_1 = require("../common-errors");
const printInfo = function (packument) {
    const versionCount = (0, record_utils_1.recordKeys)(packument.versions).length;
    const ver = (0, packument_1.tryGetLatestVersion)(packument);
    (0, assert_1.default)(ver !== undefined);
    const verInfo = packument.versions[ver];
    const license = verInfo.license || "proprietary or unlicensed";
    const displayName = verInfo.displayName;
    const description = verInfo.description || packument.description;
    const keywords = verInfo.keywords || packument.keywords;
    const homepage = verInfo.homepage;
    const dist = verInfo.dist;
    const dependencies = verInfo.dependencies;
    const latest = packument["dist-tags"]?.latest;
    let time = packument.time?.modified;
    if (!time &&
        latest &&
        packument.time !== undefined &&
        latest in packument.time)
        time = packument.time[latest];
    console.log();
    console.log(chalk_1.default.greenBright(packument.name) +
        "@" +
        chalk_1.default.greenBright(ver) +
        " | " +
        chalk_1.default.green(license) +
        " | versions: " +
        chalk_1.default.yellow(versionCount));
    console.log();
    if (displayName)
        console.log(chalk_1.default.greenBright(displayName));
    if (description)
        console.log(description);
    if (description && description.includes("\n"))
        console.log();
    if (homepage)
        console.log(chalk_1.default.cyan(homepage));
    if (keywords)
        console.log(`keywords: ${keywords.join(", ")}`);
    if (dist) {
        console.log();
        console.log("dist");
        console.log(".tarball: " + chalk_1.default.cyan(dist.tarball));
        if (dist.shasum)
            console.log(".shasum: " + chalk_1.default.yellow(dist.shasum));
        if (dist.integrity)
            console.log(".integrity: " + chalk_1.default.yellow(dist.integrity));
    }
    if (dependencies && (0, record_utils_1.recordKeys)(dependencies).length > 0) {
        console.log();
        console.log("dependencies");
        (0, record_utils_1.recordKeys)(dependencies)
            .sort()
            .forEach((n) => console.log(chalk_1.default.yellow(n) + ` ${dependencies[n]}`));
    }
    console.log();
    console.log("latest: " + chalk_1.default.greenBright(latest));
    console.log();
    console.log("published at " + chalk_1.default.yellow(time));
    console.log();
    console.log("versions:");
    for (const version in packument.versions) {
        console.log("  " + chalk_1.default.greenBright(version));
    }
};
/**
 * Makes a {@link ViewCmd} function.
 */
function makeViewCmd(parseEnv, resolveRemotePackument) {
    return async (pkg, options) => {
        // parse env
        const envResult = await parseEnv(options);
        if (envResult.isErr())
            return envResult;
        const env = envResult.value;
        // parse name
        if ((0, package_reference_1.hasVersion)(pkg)) {
            const [name] = (0, package_reference_1.splitPackageReference)(pkg);
            logger_1.default.warn("", `please do not specify a version (Write only '${name}').`);
            return (0, ts_results_es_1.Err)(new common_errors_1.PackageWithVersionError());
        }
        // verify name
        const result = await resolveRemotePackument(pkg, undefined, env.registry).orElse((error) => env.upstream
            ? resolveRemotePackument(pkg, undefined, env.upstreamRegistry)
            : (0, ts_results_es_1.Err)(error)).promise;
        if (result.isOk()) {
            printInfo(result.value.packument);
            return (0, ts_results_es_1.Ok)(undefined);
        }
        else {
            logger_1.default.error("404", `package not found: ${pkg}`);
            return result;
        }
    };
}
exports.makeViewCmd = makeViewCmd;
//# sourceMappingURL=cmd-view.js.map