"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const extra_typings_1 = require("@commander-js/extra-typings");
const pkginfo_1 = __importDefault(require("pkginfo"));
const update_notifier_1 = __importDefault(require("update-notifier"));
const cmd_remove_1 = require("./cmd-remove");
const cmd_deps_1 = require("./cmd-deps");
const cmd_login_1 = require("./cmd-login");
const logger_1 = __importDefault(require("./logger"));
const cli_parsing_1 = require("./cli-parsing");
const fetch_packument_1 = require("../services/fetch-packument");
const cmd_add_1 = require("./cmd-add");
const npmrc_auth_1 = require("../services/npmrc-auth");
const add_user_1 = require("../services/add-user");
const search_registry_1 = require("../services/search-registry");
const package_json_1 = __importDefault(require("../../package.json"));
const cmd_search_1 = require("./cmd-search");
const cmd_view_1 = require("./cmd-view");
const dependency_resolving_1 = require("../services/dependency-resolving");
const get_all_packuments_1 = require("../services/get-all-packuments");
const validators_1 = require("./validators");
const another_npm_registry_client_1 = __importDefault(require("another-npm-registry-client"));
const parse_env_1 = require("../services/parse-env");
const resolve_remote_packument_1 = require("../services/resolve-remote-packument");
const project_manifest_io_1 = require("../io/project-manifest-io");
// Composition root
const regClient = new another_npm_registry_client_1.default({ log: logger_1.default });
const loadProjectManifest = (0, project_manifest_io_1.makeProjectManifestLoader)();
const writeProjectManifest = (0, project_manifest_io_1.makeProjectManifestWriter)();
const parseEnv = (0, parse_env_1.makeParseEnvService)();
const fetchPackument = (0, fetch_packument_1.makeFetchPackumentService)(regClient);
const authNpmrc = (0, npmrc_auth_1.makeAuthNpmrcService)();
const addUser = (0, add_user_1.makeAddUserService)(regClient);
const searchRegistry = (0, search_registry_1.makeSearchRegistryService)();
const resolveRemotePackument = (0, resolve_remote_packument_1.makeResolveRemotePackumentService)(fetchPackument);
const resolveDependencies = (0, dependency_resolving_1.makeResolveDependenciesService)(resolveRemotePackument);
const getAllPackuments = (0, get_all_packuments_1.makeGetAllPackumentsService)();
const addCmd = (0, cmd_add_1.makeAddCmd)(parseEnv, resolveRemotePackument, resolveDependencies, loadProjectManifest, writeProjectManifest);
const loginCmd = (0, cmd_login_1.makeLoginCmd)(parseEnv, authNpmrc, addUser);
const searchCmd = (0, cmd_search_1.makeSearchCmd)(parseEnv, searchRegistry, getAllPackuments);
const depsCmd = (0, cmd_deps_1.makeDepsCmd)(parseEnv, resolveDependencies);
const removeCmd = (0, cmd_remove_1.makeRemoveCmd)(parseEnv, loadProjectManifest, writeProjectManifest);
const viewCmd = (0, cmd_view_1.makeViewCmd)(parseEnv, resolveRemotePackument);
// update-notifier
(0, pkginfo_1.default)(module);
const notifier = (0, update_notifier_1.default)({ pkg: package_json_1.default });
notifier.notify();
const program = (0, extra_typings_1.createCommand)()
    .version(module.exports.version)
    .option("-c, --chdir <path>", "change the working directory")
    .option("-r, --registry <url>", "specify registry url", validators_1.mustBeRegistryUrl)
    .option("-v, --verbose", "output extra debugging")
    .option("--cn", "use the China region registry")
    .option("--system-user", "auth for Windows system user")
    .option("--wsl", "auth for Windows when using WSL")
    .option("--no-upstream", "don't use upstream unity registry")
    .option("--no-color", "disable color");
/**
 * Creates a CmdOptions object by adding global options to the given
 * specific options.
 * @param specificOptions The specific options.
 */
function makeCmdOptions(specificOptions) {
    return { ...specificOptions, _global: program.opts() };
}
program
    .command("add")
    .argument("<pkg>", "Reference to the package that should be added", validators_1.mustBePackageReference)
    .argument("[otherPkgs...]", "References to additional packages that should be added", (0, cli_parsing_1.eachValue)(validators_1.mustBePackageReference))
    .aliases(["install", "i"])
    .option("-t, --test", "add package as testable")
    .option("-f, --force", "force add package if missing deps or editor version is not qualified")
    .description(`add package to manifest json
openupm add <pkg> [otherPkgs...]
openupm add <pkg>@<version> [otherPkgs...]`)
    .action(async function (pkg, otherPkgs, options) {
    const pkgs = [pkg].concat(otherPkgs);
    const addResult = await addCmd(pkgs, makeCmdOptions(options));
    if (addResult.isErr())
        process.exit(1);
});
program
    .command("remove")
    .argument("<pkg>", "Name of the package to remove", validators_1.mustBeDomainName)
    .argument("[otherPkgs...]", "Names of additional packages to remove", (0, cli_parsing_1.eachValue)(validators_1.mustBeDomainName))
    .aliases(["rm", "uninstall"])
    .description("remove package from manifest json")
    .action(async function (pkg, otherPkgs, options) {
    const pkgs = [pkg].concat(otherPkgs);
    const removeResult = await removeCmd(pkgs, makeCmdOptions(options));
    if (removeResult.isErr())
        process.exit(1);
});
program
    .command("search")
    .argument("<keyword>", "The keyword to search")
    .aliases(["s", "se", "find"])
    .description("Search package by keyword")
    .action(async function (keyword, options) {
    const searchResult = await searchCmd(keyword, makeCmdOptions(options));
    if (searchResult.isErr())
        process.exit(1);
});
program
    .command("view")
    .argument("<pkg>", "Reference to a package", validators_1.mustBePackageReference)
    .aliases(["v", "info", "show"])
    .description("view package information")
    .action(async function (pkg, options) {
    const result = await viewCmd(pkg, makeCmdOptions(options));
    if (result.isErr())
        process.exit(1);
});
program
    .command("deps")
    .argument("<pkg>", "Reference to a package", validators_1.mustBePackageReference)
    .alias("dep")
    .option("-d, --deep", "view package dependencies recursively")
    .description(`view package dependencies
openupm deps <pkg>
openupm deps <pkg>@<version>`)
    .action(async function (pkg, options) {
    const depsResult = await depsCmd(pkg, makeCmdOptions(options));
    if (depsResult.isErr())
        process.exit(1);
});
program
    .command("login")
    .aliases(["add-user", "adduser"])
    .option("-u, --username <username>", "username")
    .option("-p, --password <password>", "password")
    .option("-e, --email <email>", "email address")
    .option("--basic-auth", "use basic authentication instead of token")
    .option("--always-auth", "always auth for tarball hosted on a different domain")
    .description("authenticate with a scoped registry")
    .action(async function (options) {
    const loginResult = await loginCmd(makeCmdOptions(options));
    if (loginResult.isErr())
        process.exit(1);
});
// prompt for invalid command
program.on("command:*", function () {
    logger_1.default.warn("", `unknown command: ${program.args.join(" ")}`);
    logger_1.default.warn("", "see --help for a list of available commands");
    process.exit(1);
});
program.parse(process.argv);
// print help if no command is given
if (!program.args.length)
    program.help();
//# sourceMappingURL=index.js.map