"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.hasVersion = exports.makePackageReference = exports.splitPackageReference = exports.isPackageReference = void 0;
const domain_name_1 = require("./domain-name");
const semantic_version_1 = require("./semantic-version");
const package_url_1 = require("./package-url");
const string_utils_1 = require("../utils/string-utils");
const assert_1 = __importDefault(require("assert"));
/**
 * Checks if a string is a version-reference.
 * @param s The string.
 */
function isVersionReference(s) {
    return s === "latest" || (0, semantic_version_1.isSemanticVersion)(s) || (0, package_url_1.isPackageUrl)(s);
}
/**
 * Checks if a string is a package-reference.
 * @param s The string.
 */
function isPackageReference(s) {
    const [name, version] = (0, string_utils_1.trySplitAtFirstOccurrenceOf)(s, "@");
    return ((0, domain_name_1.isDomainName)(name) && (version === null || isVersionReference(version)));
}
exports.isPackageReference = isPackageReference;
/**
 * Split a package-reference into the name and version if present.
 * @param reference The reference.
 */
function splitPackageReference(reference) {
    const [name, version] = (0, string_utils_1.trySplitAtFirstOccurrenceOf)(reference, "@");
    return [name, version ?? undefined];
}
exports.splitPackageReference = splitPackageReference;
/**
 * Constructs a package-reference.
 * @param name The package-name. Will be validated to be a {@link DomainName}.
 * @param version Optional version-reference. Will be validated to be a
 * {@link VersionReference}.
 */
function makePackageReference(name, version) {
    (0, assert_1.default)((0, domain_name_1.isDomainName)(name), `${name} is valid package-name`);
    (0, assert_1.default)(version === undefined || isVersionReference(version), `"${version}" is valid version-reference`);
    return version !== undefined ? `${name}@${version}` : name;
}
exports.makePackageReference = makePackageReference;
/**
 * Checks if the package-reference includes a version.
 * @param reference The package-reference.
 */
function hasVersion(reference) {
    return reference.includes("@");
}
exports.hasVersion = hasVersion;
//# sourceMappingURL=package-reference.js.map