"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.addAuth = exports.tryGetAuthForRegistry = exports.tryToNpmAuth = exports.shouldAlwaysAuth = exports.tryDecodeBasicAuth = exports.encodeBasicAuth = exports.isTokenAuth = exports.isBasicAuth = void 0;
const string_utils_1 = require("../utils/string-utils");
const base64_1 = require("./base64");
const logger_1 = __importDefault(require("../cli/logger"));
/**
 * Checks if an auth-object uses basic authentication.
 * @param auth The auth-object.
 */
function isBasicAuth(auth) {
    return "_auth" in auth;
}
exports.isBasicAuth = isBasicAuth;
/**
 * Checks if an auth-object uses token authentication.
 * @param auth The auth-object.
 */
function isTokenAuth(auth) {
    return "token" in auth;
}
exports.isTokenAuth = isTokenAuth;
/**
 * Encodes a username and password using base64.
 * @param username The username.
 * @param password The password.
 */
function encodeBasicAuth(username, password) {
    return (0, base64_1.encodeBase64)(`${username}:${password}`);
}
exports.encodeBasicAuth = encodeBasicAuth;
/**
 * Decodes a base64 encoded username and password.
 * @param base64 The base64 string.
 * @returns Password/username tuple or null if the decoded string could
 * not be parsed.
 */
function tryDecodeBasicAuth(base64) {
    const text = (0, base64_1.decodeBase64)(base64);
    const [username, password] = (0, string_utils_1.trySplitAtFirstOccurrenceOf)(text, ":");
    if (password === null)
        return null;
    return [username, password];
}
exports.tryDecodeBasicAuth = tryDecodeBasicAuth;
/**
 * Checks if this auth-object should always authenticate.
 * @param auth The auth-object.
 */
function shouldAlwaysAuth(auth) {
    return auth.alwaysAuth || false;
}
exports.shouldAlwaysAuth = shouldAlwaysAuth;
/**
 * Attempts to convert a {@link UpmAuth} object to a {@link NpmAuth} object.
 * @param upmAuth The auth-object to convert.
 * @returns The converted object or null, if conversion failed.
 */
function tryToNpmAuth(upmAuth) {
    if (isTokenAuth(upmAuth)) {
        return {
            token: upmAuth.token,
            alwaysAuth: shouldAlwaysAuth(upmAuth),
        };
    }
    else if (isBasicAuth(upmAuth)) {
        const decoded = tryDecodeBasicAuth(upmAuth._auth);
        if (decoded === null)
            return null;
        const [username, password] = decoded;
        return {
            username,
            password: (0, base64_1.encodeBase64)(password),
            email: upmAuth.email,
            alwaysAuth: shouldAlwaysAuth(upmAuth),
        };
    }
    return null;
}
exports.tryToNpmAuth = tryToNpmAuth;
/**
 * Attempts to get the {@link NpmAuth} information for a specific registry
 * from a {@link UPMConfig} object.
 * @param upmConfig The config.
 * @param registry The registry.
 * @returns The auth information or null if the registry does not exist
 * in the config.
 */
function tryGetAuthForRegistry(upmConfig, registry) {
    const upmAuth = upmConfig.npmAuth?.[registry] ||
        // As a backup search for the registry with trailing slash
        upmConfig.npmAuth?.[registry + "/"];
    if (upmAuth === undefined)
        return null;
    const npmAuth = tryToNpmAuth(upmAuth);
    if (npmAuth === null) {
        logger_1.default.warn("env.auth", `failed to parse auth info for ${registry} in .upmconfig.toml: missing token or _auth fields`);
    }
    return npmAuth;
}
exports.tryGetAuthForRegistry = tryGetAuthForRegistry;
/**
 * Adds authentication information to a {@link UPMConfig} object.
 * @param registry The registry under which to add the auth-information.
 * @param auth The auth-information.
 * @param config The config object that should be extended.
 * @returns An extended {@link UPMConfig} object.
 */
function addAuth(registry, auth, config) {
    const authContainer = config.npmAuth || {};
    return { npmAuth: { ...authContainer, [registry]: auth } };
}
exports.addAuth = addAuth;
//# sourceMappingURL=upm-config.js.map