"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tryGetDirectoriesIn = exports.tryWriteTextToFile = exports.tryReadTextFromFile = exports.IOError = exports.NotFoundError = void 0;
const ts_results_es_1 = require("ts-results-es");
const promises_1 = __importDefault(require("fs/promises"));
const ts_custom_error_1 = require("ts-custom-error");
const error_type_guards_1 = require("../utils/error-type-guards");
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
/**
 * Error for when a file or directory did not exist.
 */
class NotFoundError extends ts_custom_error_1.CustomError {
    constructor(
    /**
     * The path to the missing file.
     */
    path) {
        super("An IO operation was performed on a file or directory which does not exist.");
        this.path = path;
        this._class = "NotFoundError";
    }
}
exports.NotFoundError = NotFoundError;
/**
 * Generic IO error for when interacting with the file-system failed.
 */
class IOError extends ts_custom_error_1.CustomError {
    constructor(
    /**
     * The actual error that caused the failure.
     */
    cause) {
        super("An interaction with the file-system caused an error.", { cause });
        this._class = "CustomError";
    }
}
exports.IOError = IOError;
/**
 * Attempts to read the content of a text file.
 * @param path The path to the file.
 */
function tryReadTextFromFile(path) {
    return new ts_results_es_1.AsyncResult(ts_results_es_1.Result.wrapAsync(() => promises_1.default.readFile(path, { encoding: "utf8" }))).mapErr((error) => {
        (0, error_type_guards_1.assertIsNodeError)(error);
        if (error.code === "ENOENT")
            return new NotFoundError(path);
        return new IOError(error);
    });
}
exports.tryReadTextFromFile = tryReadTextFromFile;
/**
 * Attempts to overwrite the content of a text file.
 * @param filePath The path to the file.
 * @param content The content to write.
 */
function tryWriteTextToFile(filePath, content) {
    return new ts_results_es_1.AsyncResult(ts_results_es_1.Result.wrapAsync(() => fs_extra_1.default.ensureDir(path_1.default.dirname(filePath))))
        .andThen(() => ts_results_es_1.Result.wrapAsync(() => promises_1.default.writeFile(filePath, content)))
        .mapErr((error) => {
        (0, error_type_guards_1.assertIsNodeError)(error);
        return new IOError(error);
    });
}
exports.tryWriteTextToFile = tryWriteTextToFile;
/**
 * Attempts to get the names of all directories in a directory.
 * @param directoryPath The directories name.
 */
function tryGetDirectoriesIn(directoryPath) {
    return new ts_results_es_1.AsyncResult(ts_results_es_1.Result.wrapAsync(() => promises_1.default.readdir(directoryPath, { withFileTypes: true })))
        .map((entries) => entries.filter((it) => it.isDirectory()))
        .map((directories) => directories.map((it) => it.name))
        .mapErr((error) => {
        (0, error_type_guards_1.assertIsNodeError)(error);
        if (error.code === "ENOENT")
            return new NotFoundError(directoryPath);
        return new IOError(error);
    });
}
exports.tryGetDirectoriesIn = tryGetDirectoriesIn;
//# sourceMappingURL=file-io.js.map