"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tryLoadProjectVersion = exports.tryCreateProjectVersionTxt = void 0;
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const ts_results_es_1 = require("ts-results-es");
const yaml_1 = __importDefault(require("yaml"));
const common_errors_1 = require("../common-errors");
const file_io_1 = require("./file-io");
const error_type_guards_1 = require("../utils/error-type-guards");
function projectVersionTxtPathFor(projectDirPath) {
    return path_1.default.join(projectDirPath, "ProjectSettings", "ProjectVersion.txt");
}
/**
 * Creates a ProjectVersion.txt file for a Unity project.
 * Nothing besides m_EditorVersion is specified.
 * @param projectDirPath The projects root folder.
 * @param version The editor-version to use.
 */
function tryCreateProjectVersionTxt(projectDirPath, version) {
    return new ts_results_es_1.AsyncResult(ts_results_es_1.Result.wrapAsync(async () => {
        const projectSettingsDir = path_1.default.join(projectDirPath, "ProjectSettings");
        await fs_extra_1.default.mkdirp(projectSettingsDir);
        const data = `m_EditorVersion: ${version}`;
        await fs_extra_1.default.writeFile(path_1.default.join(projectSettingsDir, "ProjectVersion.txt"), data);
    }));
}
exports.tryCreateProjectVersionTxt = tryCreateProjectVersionTxt;
/**
 * Attempts to load a projects editor-version from ProjectVersion.txt.
 * @param projectDirPath The path to the projects root directory.
 */
function tryLoadProjectVersion(projectDirPath) {
    const filePath = projectVersionTxtPathFor(projectDirPath);
    return (0, file_io_1.tryReadTextFromFile)(filePath)
        .andThen((text) => ts_results_es_1.Result.wrap(() => yaml_1.default.parse(text)).mapErr((error) => {
        (0, error_type_guards_1.assertIsError)(error);
        return new common_errors_1.FileParseError(filePath, "ProjectVersion.txt", error);
    }))
        .andThen((content) => {
        if (!(typeof content === "object" &&
            content !== null &&
            "m_EditorVersion" in content &&
            typeof content.m_EditorVersion === "string"))
            return (0, ts_results_es_1.Err)(new common_errors_1.FileParseError(filePath, "Project-version"));
        return (0, ts_results_es_1.Ok)(content.m_EditorVersion);
    });
}
exports.tryLoadProjectVersion = tryLoadProjectVersion;
//# sourceMappingURL=project-version-io.js.map