"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tryGetEditorInstallPath = exports.tryGetHomePath = exports.OSNotSupportedError = exports.VersionNotSupportedOnOsError = void 0;
const ts_results_es_1 = require("ts-results-es");
const upm_config_io_1 = require("./upm-config-io");
const env_util_1 = require("../utils/env-util");
const os_1 = __importDefault(require("os"));
const ts_custom_error_1 = require("ts-custom-error");
const editor_version_1 = require("../domain/editor-version");
const common_errors_1 = require("../common-errors");
/**
 * Error for when a specific OS does not support a specific editor-version.
 */
class VersionNotSupportedOnOsError extends ts_custom_error_1.CustomError {
    constructor(
    /**
     * The version that is not supported.
     */
    version, 
    /**
     * Name of the OS, which does not support the version.
     */
    os) {
        super();
        this.version = version;
        this.os = os;
        this._class = "VersionNotSupportedOnOsError";
    }
}
exports.VersionNotSupportedOnOsError = VersionNotSupportedOnOsError;
/**
 * Error for when Unity does not support an OS.
 */
class OSNotSupportedError extends ts_custom_error_1.CustomError {
    constructor(
    /**
     * Name of the unsupported OS.
     */
    os) {
        super();
        this.os = os;
        this._class = "OSNotSupportedError";
    }
}
exports.OSNotSupportedError = OSNotSupportedError;
/**
 * Attempts to get the current users home-directory.
 */
function tryGetHomePath() {
    const homePath = (0, env_util_1.tryGetEnv)("USERPROFILE") ?? (0, env_util_1.tryGetEnv)("HOME");
    if (homePath === null)
        return (0, ts_results_es_1.Err)(new upm_config_io_1.RequiredEnvMissingError("USERPROFILE", "HOME"));
    return (0, ts_results_es_1.Ok)(homePath);
}
exports.tryGetHomePath = tryGetHomePath;
const firstSupportedVersionOnLinux = (0, editor_version_1.makeEditorVersion)(2019, 2);
/**
 * The first version which supports packages.
 * (The package-manager was officially added in 2017.
 * @see https://blog.unity.com/technology/project-management-is-evolving-unity-package-manager-overview
 */
const firstSupportedEditorVersion = (0, editor_version_1.makeEditorVersion)(2018, 1);
/**
 * Attempts to get the default installation path of a specific Unity editor.
 * Returned paths do not include a trailing slash.
 * @param editorVersion The version-string for the editor e.g. 2022.1.3f1.
 */
function tryGetEditorInstallPath(editorVersion) {
    const isVersionSupported = (0, editor_version_1.compareEditorVersion)(editorVersion, firstSupportedEditorVersion) >= 0;
    if (!isVersionSupported)
        return (0, ts_results_es_1.Err)(new common_errors_1.EditorVersionNotSupportedError(editorVersion));
    const platform = os_1.default.platform();
    const versionString = (0, editor_version_1.stringifyEditorVersion)(editorVersion);
    if (platform === "win32")
        return (0, ts_results_es_1.Ok)(`C:\\Program Files\\Unity\\Hub\\Editor\\${versionString}`);
    else if (platform === "linux") {
        const versionSupportsLinux = (0, editor_version_1.compareEditorVersion)(editorVersion, firstSupportedVersionOnLinux) >= 0;
        if (versionSupportsLinux)
            return (0, ts_results_es_1.Ok)(`~/Unity/Hub/Editor/${versionString}`);
        else
            return (0, ts_results_es_1.Err)(new VersionNotSupportedOnOsError(editorVersion, platform));
    }
    else if (platform === "darwin")
        return (0, ts_results_es_1.Ok)(`/Applications/Unity/Hub/Editor/${versionString}`);
    return (0, ts_results_es_1.Err)(new OSNotSupportedError(platform));
}
exports.tryGetEditorInstallPath = tryGetEditorInstallPath;
//# sourceMappingURL=special-paths.js.map