"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tryStoreUpmAuth = exports.trySaveUpmConfig = exports.tryLoadUpmConfig = exports.tryGetUpmConfigDir = exports.RequiredEnvMissingError = void 0;
const path_1 = __importDefault(require("path"));
const toml_1 = __importDefault(require("@iarna/toml"));
const logger_1 = __importDefault(require("../cli/logger"));
const upm_config_1 = require("../domain/upm-config");
const ts_custom_error_1 = require("ts-custom-error");
const ts_results_es_1 = require("ts-results-es");
const file_io_1 = require("./file-io");
const env_util_1 = require("../utils/env-util");
const special_paths_1 = require("./special-paths");
const data_parsing_1 = require("../utils/data-parsing");
const wls_1 = require("./wls");
const configFileName = ".upmconfig.toml";
class RequiredEnvMissingError extends ts_custom_error_1.CustomError {
    constructor(...keyNames) {
        super(`Env was required to contain a value for one of the following keys, but all were missing: ${keyNames
            .map((keyName) => `"${keyName}"`)
            .join(", ")}.`);
        this._class = "RequiredEnvMissingError";
    }
}
exports.RequiredEnvMissingError = RequiredEnvMissingError;
/**
 * Gets the path to directory in which the upm config is stored.
 * @param wsl Whether WSL should be treated as Windows.
 * @param systemUser Whether to authenticate as a Windows system-user.
 */
const tryGetUpmConfigDir = (wsl, systemUser) => {
    const systemUserSubPath = "Unity/config/ServiceAccounts";
    if (wsl) {
        if (systemUser)
            return (0, wls_1.tryGetWslPath)("ALLUSERSPROFILE").map((it) => path_1.default.join(it, systemUserSubPath));
        return (0, wls_1.tryGetWslPath)("USERPROFILE");
    }
    if (systemUser) {
        const profilePath = (0, env_util_1.tryGetEnv)("ALLUSERSPROFILE");
        if (profilePath === null)
            return (0, ts_results_es_1.Err)(new RequiredEnvMissingError("ALLUSERSPROFILE")).toAsyncResult();
        return (0, ts_results_es_1.Ok)(path_1.default.join(profilePath, systemUserSubPath)).toAsyncResult();
    }
    return (0, special_paths_1.tryGetHomePath)().toAsyncResult();
};
exports.tryGetUpmConfigDir = tryGetUpmConfigDir;
/**
 * Attempts to load the upm config.
 * @param configDir The directory from which to load the config.
 * @returns The config or null if not found.
 */
const tryLoadUpmConfig = (configDir) => {
    const configPath = path_1.default.join(configDir, configFileName);
    return ((0, file_io_1.tryReadTextFromFile)(configPath)
        .andThen(data_parsing_1.tryParseToml)
        // TODO: Actually validate
        .map((toml) => toml)
        .orElse((error) => error instanceof file_io_1.NotFoundError ? (0, ts_results_es_1.Ok)(null) : (0, ts_results_es_1.Err)(error)));
};
exports.tryLoadUpmConfig = tryLoadUpmConfig;
/**
 * Save the upm config.
 * @param config The config to save.
 * @param configDir The directory in which to save the config.
 * @return The path to which the file was saved.
 */
const trySaveUpmConfig = (config, configDir) => {
    const configPath = path_1.default.join(configDir, configFileName);
    const content = toml_1.default.stringify(config);
    return (0, file_io_1.tryWriteTextToFile)(configPath, content).map(() => configPath);
};
exports.trySaveUpmConfig = trySaveUpmConfig;
/**
 * Stores authentication information in the projects upm config.
 */
const tryStoreUpmAuth = function (configDir, registry, auth) {
    return (0, exports.tryLoadUpmConfig)(configDir)
        .map((maybeConfig) => maybeConfig || {})
        .map((config) => (0, upm_config_1.addAuth)(registry, auth, config))
        .andThen((config) => (0, exports.trySaveUpmConfig)(config, configDir))
        .map((configPath) => logger_1.default.notice("config", "saved unity config at " + configPath));
};
exports.tryStoreUpmAuth = tryStoreUpmAuth;
//# sourceMappingURL=upm-config-io.js.map