"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.pickMostFixable = exports.tryResolveFromCache = exports.tryResolveFromPackument = exports.VersionNotFoundError = exports.NoVersionsError = void 0;
const packument_1 = require("./domain/packument");
const record_utils_1 = require("./utils/record-utils");
const packument_cache_1 = require("./packument-cache");
const ts_custom_error_1 = require("ts-custom-error");
const ts_results_es_1 = require("ts-results-es");
const common_errors_1 = require("./common-errors");
/**
 * Error for when a packument with the searched name was found, but it
 * had no versions.
 */
class NoVersionsError extends ts_custom_error_1.CustomError {
    constructor() {
        super("A packument contained no versions");
        this._class = "NoVersionsError";
    }
}
exports.NoVersionsError = NoVersionsError;
/**
 * Error for when a packument with the searched name was found, but a specific
 * requested version did not exist.
 */
class VersionNotFoundError extends ts_custom_error_1.CustomError {
    constructor(
    /**
     * The version that was requested.
     */
    requestedVersion, 
    /**
     * A list of available versions.
     */
    availableVersions) {
        super("The requested version was not in the packument.");
        this.requestedVersion = requestedVersion;
        this.availableVersions = availableVersions;
        this._class = "VersionNotFoundError";
    }
}
exports.VersionNotFoundError = VersionNotFoundError;
/**
 * Attempts to resolve a specific version from a packument.
 * @param packument The packument to search.
 * @param requestedVersion The requested version.
 * @param source The source from which the packument was resolved.
 */
function tryResolveFromPackument(packument, requestedVersion, source) {
    const availableVersions = (0, record_utils_1.recordKeys)(packument.versions);
    if (availableVersions.length === 0)
        return (0, ts_results_es_1.Err)(new NoVersionsError());
    // Find the latest version
    if (requestedVersion === undefined || requestedVersion === "latest") {
        let latestVersion = (0, packument_1.tryGetLatestVersion)(packument);
        if (latestVersion === undefined)
            latestVersion = availableVersions.at(-1);
        return (0, ts_results_es_1.Ok)({
            packument,
            source,
            packumentVersion: packument.versions[latestVersion],
        });
    }
    // Find a specific version
    if (!availableVersions.includes(requestedVersion))
        return (0, ts_results_es_1.Err)(new VersionNotFoundError(requestedVersion, availableVersions));
    return (0, ts_results_es_1.Ok)({
        packument,
        source,
        packumentVersion: packument.versions[requestedVersion],
    });
}
exports.tryResolveFromPackument = tryResolveFromPackument;
/**
 * Attempts to resolve a packument-version from the cache.
 * @param cache The cache to search.
 * @param source The source from which to resolve.
 * @param packumentName The name of the packument to resolve.
 * @param requestedVersion The requested version.
 */
function tryResolveFromCache(cache, source, packumentName, requestedVersion) {
    const cachedPackument = (0, packument_cache_1.tryGetFromCache)(cache, source, packumentName);
    if (cachedPackument === null)
        return (0, ts_results_es_1.Err)(new common_errors_1.PackumentNotFoundError());
    return tryResolveFromPackument(cachedPackument, requestedVersion, source);
}
exports.tryResolveFromCache = tryResolveFromCache;
/**
 * Compares two resolve-failures to check which is more fixable.
 * @param a The first failure.
 * @param b The second failure.
 * @returns The more fixable failure.
 */
function pickMostFixable(a, b) {
    // Anything is more fixable than packument-not-found
    if (a.error instanceof common_errors_1.PackumentNotFoundError &&
        !(b.error instanceof common_errors_1.PackumentNotFoundError))
        return b;
    else if (b.error instanceof common_errors_1.PackumentNotFoundError &&
        !(a.error instanceof common_errors_1.PackumentNotFoundError))
        return a;
    return a;
}
exports.pickMostFixable = pickMostFixable;
//# sourceMappingURL=packument-resolving.js.map