"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeResolveDependenciesService = void 0;
const domain_name_1 = require("../domain/domain-name");
const semantic_version_1 = require("../domain/semantic-version");
const logger_1 = __importDefault(require("../cli/logger"));
const package_reference_1 = require("../domain/package-reference");
const packument_cache_1 = require("../packument-cache");
const packument_resolving_1 = require("../packument-resolving");
const registry_url_1 = require("../domain/registry-url");
const record_utils_1 = require("../utils/record-utils");
const assert_1 = __importDefault(require("assert"));
const common_errors_1 = require("../common-errors");
/**
 * Makes a {@link ResolveDependenciesService} function.
 */
function makeResolveDependenciesService(resolveRemotePackument) {
    return async (registry, upstreamRegistry, name, version, deep) => {
        // a list of pending dependency {name, version}
        const pendingList = [{ name, version }];
        // a list of processed dependency {name, version}
        const processedList = Array.of();
        // a list of dependency entry exists on the registry
        const depsValid = Array.of();
        // a list of dependency entry doesn't exist on the registry
        const depsInvalid = Array.of();
        // cached dict
        let packumentCache = packument_cache_1.emptyPackumentCache;
        async function tryResolveFromRegistry(registry, packumentName, version) {
            // First try cache
            const cacheResult = (0, packument_resolving_1.tryResolveFromCache)(packumentCache, registry.url, packumentName, version);
            if (cacheResult.isOk())
                return cacheResult;
            // Then registry
            return await resolveRemotePackument(packumentName, version, registry)
                .promise;
        }
        while (pendingList.length > 0) {
            // NOTE: Guaranteed defined because of while loop logic
            const entry = pendingList.shift();
            const isProcessed = processedList.some((x) => x.name === entry.name && x.version === entry.version);
            if (!isProcessed) {
                // add entry to processed list
                processedList.push(entry);
                const isInternal = (0, domain_name_1.isInternalPackage)(entry.name);
                const isSelf = entry.name === name;
                let isUpstream = false;
                let resolvedVersion = entry.version;
                if (!isInternal) {
                    // First primary registry
                    let resolveResult = await tryResolveFromRegistry(registry, entry.name, entry.version);
                    // Then upstream registry
                    if (resolveResult.isErr()) {
                        const upstreamResult = await tryResolveFromRegistry(upstreamRegistry, entry.name, entry.version);
                        if (upstreamResult.isOk())
                            resolveResult = upstreamResult;
                        else
                            resolveResult = (0, packument_resolving_1.pickMostFixable)(resolveResult, upstreamResult);
                    }
                    // If none resolved successfully, log the most fixable failure
                    if (resolveResult.isErr()) {
                        if (resolveResult.error instanceof common_errors_1.PackumentNotFoundError) {
                            logger_1.default.warn("404", `package not found: ${entry.name}`);
                        }
                        else if (resolveResult.error instanceof packument_resolving_1.VersionNotFoundError) {
                            const versionList = [...resolveResult.error.availableVersions]
                                .reverse()
                                .join(", ");
                            logger_1.default.warn("404", `version ${resolveResult.error.requestedVersion} is not a valid choice of ${versionList}`);
                        }
                        depsInvalid.push({
                            name: entry.name,
                            self: isSelf,
                            reason: resolveResult.error,
                        });
                        continue;
                    }
                    // Packument was resolved successfully
                    isUpstream = resolveResult.value.source === registry_url_1.unityRegistryUrl;
                    resolvedVersion = resolveResult.value.packumentVersion.version;
                    packumentCache = (0, packument_cache_1.addToCache)(packumentCache, resolveResult.value.source, resolveResult.value.packument);
                    // add dependencies to pending list
                    if (isSelf || deep) {
                        const deps = (0, record_utils_1.recordEntries)(resolveResult.value.packumentVersion["dependencies"] || {}).map((x) => {
                            return {
                                name: x[0],
                                version: x[1],
                            };
                        });
                        deps.forEach((x) => pendingList.push(x));
                    }
                }
                // We can safely assert this. entry.version can only not be a semantic
                // version for the initial input, but then it should not be internal
                // and thus resolve to a semantic version.
                (0, assert_1.default)(resolvedVersion !== undefined);
                (0, assert_1.default)((0, semantic_version_1.isSemanticVersion)(resolvedVersion));
                depsValid.push({
                    name: entry.name,
                    version: resolvedVersion,
                    internal: isInternal,
                    upstream: isUpstream,
                    self: isSelf,
                });
                logger_1.default.verbose("dependency", `${(0, package_reference_1.makePackageReference)(entry.name, resolvedVersion)} ${isInternal ? "[internal] " : ""}${isUpstream ? "[upstream]" : ""}`);
            }
        }
        return [depsValid, depsInvalid];
    };
}
exports.makeResolveDependenciesService = makeResolveDependenciesService;
//# sourceMappingURL=dependency-resolving.js.map