"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeParseEnvService = void 0;
const logger_1 = __importDefault(require("../cli/logger"));
const chalk_1 = __importDefault(require("chalk"));
const upm_config_io_1 = require("../io/upm-config-io");
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const registry_url_1 = require("../domain/registry-url");
const upm_config_1 = require("../domain/upm-config");
const common_errors_1 = require("../common-errors");
const ts_results_es_1 = require("ts-results-es");
const project_version_io_1 = require("../io/project-version-io");
const file_io_1 = require("../io/file-io");
const env_util_1 = require("../utils/env-util");
const editor_version_1 = require("../domain/editor-version");
function determineCwd(options) {
    const cwd = options._global.chdir !== undefined
        ? path_1.default.resolve(options._global.chdir)
        : process.cwd();
    if (!fs_1.default.existsSync(cwd))
        return (0, ts_results_es_1.Err)(new file_io_1.NotFoundError(cwd));
    return (0, ts_results_es_1.Ok)(cwd);
}
function determineWsl(options) {
    return options._global.wsl === true;
}
function determinePrimaryRegistry(options, upmConfig) {
    const url = options._global.registry !== undefined
        ? (0, registry_url_1.coerceRegistryUrl)(options._global.registry)
        : options._global.cn === true
            ? (0, registry_url_1.makeRegistryUrl)("https://package.openupm.cn")
            : (0, registry_url_1.makeRegistryUrl)("https://package.openupm.com");
    const auth = upmConfig !== null ? (0, upm_config_1.tryGetAuthForRegistry)(upmConfig, url) : null;
    return { url, auth };
}
function determineUpstreamRegistry(options) {
    const url = options._global.cn === true
        ? (0, registry_url_1.makeRegistryUrl)("https://packages.unity.cn")
        : (0, registry_url_1.makeRegistryUrl)("https://packages.unity.com");
    return { url, auth: null };
}
function determineLogLevel(options) {
    return options._global.verbose ? "verbose" : "notice";
}
function determineUseColor(options) {
    return options._global.color !== false && (0, env_util_1.tryGetEnv)("NODE_ENV") !== "test";
}
function determineUseUpstream(options) {
    return options._global.upstream !== false;
}
function determineIsSystemUser(options) {
    return options._global.systemUser === true;
}
/**
 * Creates a {@link ParseEnvService} function.
 */
function makeParseEnvService() {
    return async (options) => {
        // log level
        logger_1.default.level = determineLogLevel(options);
        // color
        const useColor = determineUseColor(options);
        if (!useColor) {
            chalk_1.default.level = 0;
            logger_1.default.disableColor();
        }
        // upstream
        const upstream = determineUseUpstream(options);
        // region cn
        if (options._global.cn === true)
            logger_1.default.notice("region", "cn");
        // auth
        const systemUser = determineIsSystemUser(options);
        const wsl = determineWsl(options);
        // registries
        const upmConfigResult = await (0, upm_config_io_1.tryGetUpmConfigDir)(wsl, systemUser).andThen(upm_config_io_1.tryLoadUpmConfig).promise;
        if (upmConfigResult.isErr())
            return upmConfigResult;
        const upmConfig = upmConfigResult.value;
        const registry = determinePrimaryRegistry(options, upmConfig);
        const upstreamRegistry = determineUpstreamRegistry(options);
        // cwd
        const cwdResult = determineCwd(options);
        if (cwdResult.isErr()) {
            logger_1.default.error("env", `can not resolve path ${cwdResult.error.path}`);
            return cwdResult;
        }
        const cwd = cwdResult.value;
        // editor version
        const projectVersionLoadResult = await (0, project_version_io_1.tryLoadProjectVersion)(cwd).promise;
        if (projectVersionLoadResult.isErr()) {
            if (projectVersionLoadResult.error instanceof file_io_1.NotFoundError)
                logger_1.default.warn("ProjectVersion", `can not locate ProjectVersion.text at path ${projectVersionLoadResult.error.path}`);
            else if (projectVersionLoadResult.error instanceof common_errors_1.FileParseError)
                logger_1.default.error("ProjectVersion", "ProjectVersion.txt could not be parsed for editor-version!");
            return projectVersionLoadResult;
        }
        const unparsedEditorVersion = projectVersionLoadResult.value;
        const parsedEditorVersion = (0, editor_version_1.tryParseEditorVersion)(unparsedEditorVersion);
        const editorVersion = parsedEditorVersion !== null && (0, editor_version_1.isRelease)(parsedEditorVersion)
            ? parsedEditorVersion
            : unparsedEditorVersion;
        return (0, ts_results_es_1.Ok)({
            cwd,
            editorVersion,
            registry,
            systemUser,
            upstream,
            upstreamRegistry,
            wsl,
        });
    };
}
exports.makeParseEnvService = makeParseEnvService;
//# sourceMappingURL=parse-env.js.map